package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ExportDiagnoseDTO;
import com.diagbot.entity.KlDiagnoseDetail;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.KlDiagnoseDetailServiceImpl;
import com.diagbot.util.*;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.compress.utils.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 诊断依据明细表 服务实现类
 * </p>
 *
 * @author zhoutg
 * @since 2021-06-10
 */
@Service
public class KlDiagnoseDetailFacade extends KlDiagnoseDetailServiceImpl {

    @Autowired
    KlDiagnoseDetailFacade klDiagnoseDetailFacade;

    /**
     * 导出诊断依据【初版】
     */
    public void exportDiagnose() {
        // 获取全部数据、分组
        List<KlDiagnoseDetail> klDiagnoseDetailList = klDiagnoseDetailFacade.list(new QueryWrapper<KlDiagnoseDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey()));
        Map<String, List<KlDiagnoseDetail>> diagnoseDetailMap = EntityUtil.makeEntityListMap(klDiagnoseDetailList, "disName");
        // TODO 获取公式，分隔编码，替换编码，组装导出数据  Map<String, List<ExportDiagnoseDTO>>
        Map<String, List<ExportDiagnoseDTO>> diagnoseDetails = processDiagnoseDetail(diagnoseDetailMap);
        System.out.println();


        // TODO 导出数据
    }


    public Map<String, List<ExportDiagnoseDTO>> processDiagnoseDetail(Map<String, List<KlDiagnoseDetail>> diagnoseDetailMap){
        Map<String, List<ExportDiagnoseDTO>> data = new LinkedHashMap<>();
        for(Map.Entry<String, List<KlDiagnoseDetail>> kd : diagnoseDetailMap.entrySet()){
            String sheetName = kd.getKey();
            List<KlDiagnoseDetail> datats = kd.getValue();
            removeKlDiagnoseDetail(datats);
            List<ExportDiagnoseDTO> eds = Lists.newArrayList();
            for (KlDiagnoseDetail kdl:datats) {
                ExportDiagnoseDTO exportDiagnoseDTO = new ExportDiagnoseDTO();
                BeanUtil.copyProperties(kdl, exportDiagnoseDTO);
                ReflectUtil.setProperty(exportDiagnoseDTO, "type", kdl.getType().toString());
                ReflectUtil.setProperty(exportDiagnoseDTO, "orderNo", kdl.getCode());
                if(kdl.getCode().startsWith("3.")){
                    exportDiagnoseDTO.setRule(kdl.getRelation());
                }else {
                    exportDiagnoseDTO.setRule(kdl.getStandard());
                }
                exportDiagnoseDTO.setEq(kdl.getMidResult());
                eds.add(exportDiagnoseDTO);
            }
            data.put(sheetName,eds);
        }
        return data;
    }

    /**
     * 移除公式编码中没有的数据
     * @param klDiagnoseDetails
     */
    public void removeKlDiagnoseDetail(List<KlDiagnoseDetail> klDiagnoseDetails){
        Map<String, List<String>> allFormula = getAllFormula(klDiagnoseDetails);
        List<String> regexData = getCodes(allFormula);
        Iterator<KlDiagnoseDetail> iterator = klDiagnoseDetails.iterator();
        while (iterator.hasNext()){
            KlDiagnoseDetail next = iterator.next();
            String code = next.getCode();
            if(!regexData.contains(code)){
                iterator.remove();
            }
        }
    }


    /**
     * 获取每个诊断依据的公式
     * @param klDiagnoseDetails
     * @return
     */
    private Map<String,List<String>> getAllFormula(List<KlDiagnoseDetail> klDiagnoseDetails){
        Map<String,List<String>> typeFormulaMap = new HashMap<>();
        for (KlDiagnoseDetail klDiagnoseDetail : klDiagnoseDetails) {
            Integer type = klDiagnoseDetail.getType();
            String formula = klDiagnoseDetail.getFormula();
            if((type == 91 || type == 92 || type == 93)&& StringUtil.isNotBlank(formula)){
                if(typeFormulaMap.containsKey(type.toString())){
                    List<String> formulas = typeFormulaMap.get(type.toString());
                    formulas.add(formula);
                    typeFormulaMap.put(type.toString(),formulas);
                }else {
                    List<String> formulas = Lists.newArrayList();
                    formulas.add(formula);
                    typeFormulaMap.put(type.toString(),formulas);
                }
            }
        }
        return typeFormulaMap;
    }

    /**
     * 获取公式中的所有编码
     * @param getAllFormula
     * @return
     */
    private List<String> getCodes(Map<String,List<String>> getAllFormula) {
        StringBuffer sb = new StringBuffer();
        String pateern = "([1-9]\\d*\\.?\\d*)|(0\\.\\d*[1-9])";
        if(MapUtils.isNotEmpty(getAllFormula)){
            getAllFormula.forEach((x,y) ->{
                String collect = y.stream().collect(Collectors.joining(","));
                sb.append(collect);
            });
        }
        //获取公式中的编码
        return RegexUtil.getRegexDatas(sb.toString(), pateern);
    }

}
